package web.multitask.trismegistoservices.api;

import org.json.JSONObject;
import lombok.AllArgsConstructor;
import org.springframework.core.io.Resource;
import org.springframework.http.HttpHeaders;
import org.springframework.http.ResponseEntity;
import org.springframework.web.bind.annotation.CrossOrigin;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;
import web.multitask.trismegistoservices.services.ExcelService;
import web.multitask.trismegistoservices.utils.CommonUtils;

@RestController
@CrossOrigin(origins = "*")
@RequestMapping("excel")
@AllArgsConstructor
public class ExcelApi {

    ExcelService excelService;
    CommonUtils commonUtils;

    @PostMapping("/public/generate")
    public ResponseEntity<?> generateExcel(@RequestBody String json) {
        JSONObject jsonBody = new JSONObject(json);
        byte[] excelByte = excelService.generateExcel(jsonBody);
        if (jsonBody.optBoolean("base64", false)) {
            JSONObject response = new JSONObject();
            response.put("file_name", jsonBody.optString("file_name", "no_name.xlsx"));
            response.put("base64", commonUtils.byteToBase64(excelByte));
            response.put("message", "OK");
            response.put("status", true);
            return ResponseEntity.ok().body(response.toMap());
        } else {
            Resource resource = excelService.convertByteToResource(excelByte, jsonBody.optString("file_name", "no_name.xlsx"));
            HttpHeaders headers = new HttpHeaders();
            headers.add(HttpHeaders.CONTENT_DISPOSITION, "attachment; filename=" + jsonBody.optString("file_name", "no_name.xlsx"));
            return ResponseEntity.ok()
                    .headers(headers)
                    .contentLength(excelByte.length)
                    .body(resource);
        }
    }
}