package web.multitask.trismegistoservices.services;

import java.awt.Color;
import java.io.ByteArrayOutputStream;
import org.apache.poi.ss.usermodel.BorderStyle;
import org.apache.poi.ss.usermodel.FillPatternType;
import org.apache.poi.ss.usermodel.Font;
import org.apache.poi.ss.usermodel.HorizontalAlignment;
import org.apache.poi.ss.usermodel.VerticalAlignment;
import org.apache.poi.xssf.usermodel.XSSFCell;
import org.apache.poi.xssf.usermodel.XSSFCellStyle;
import org.apache.poi.xssf.usermodel.XSSFColor;
import org.apache.poi.xssf.usermodel.XSSFRow;
import org.apache.poi.xssf.usermodel.XSSFSheet;
import org.apache.poi.xssf.usermodel.XSSFWorkbook;
import org.apache.poi.xssf.usermodel.extensions.XSSFCellBorder.BorderSide;
import org.json.JSONArray;
import org.json.JSONObject;
import org.springframework.core.io.ByteArrayResource;
import org.springframework.core.io.Resource;
import org.springframework.stereotype.Service;

@Service
public class ExcelService {

    public byte[] generateExcel(JSONObject json) {
        ByteArrayOutputStream outByteStream = new ByteArrayOutputStream();
        XSSFWorkbook workbook = new XSSFWorkbook();
        createSheet(workbook, json);
        try {
            workbook.write(outByteStream);
            workbook.close();
            return outByteStream.toByteArray();
        } catch (Exception e) {
            System.out.println(e.getMessage());
            return null;
        }
    }

    private void createSheet(XSSFWorkbook workbook, JSONObject json) {
        if (json.optJSONArray("sheetArray", new JSONArray()).isEmpty()) {
            XSSFSheet sheet = workbook.createSheet(json.optString("sheet_name", "no_name"));
            buildSheet(workbook, sheet, json);
        }else{
            JSONArray sheetArray = json.optJSONArray("sheetArray", new JSONArray());
            for(int i = 0; i < sheetArray.length(); i++){
                JSONObject sheetObject = sheetArray.optJSONObject(i);
                XSSFSheet sheet = workbook.createSheet(sheetObject.optString("sheet_name", "no_name"));
                buildSheet(workbook, sheet, sheetObject);
            }
        }
    }

    private void buildSheet(XSSFWorkbook workbook, XSSFSheet sheet, JSONObject json) {

        final XSSFCellStyle defaultStyle = createCellStyle(workbook, new JSONObject());
        
        JSONArray identifiers = json.optJSONArray("identifiers", new JSONArray());
        JSONArray data = json.optJSONArray("data", new JSONArray());
        JSONArray styles = json.optJSONArray("styles", new JSONArray());
        for (int i = 0; i < data.length(); i++) {
            XSSFRow row = sheet.createRow(i);
            JSONObject row_data = data.optJSONObject(i);
            for (int j = 0; j < identifiers.length(); j++) {
                XSSFCell cell = row.createCell(j);
                JSONArray styleArray = styles.optJSONArray(i, new JSONArray());
                JSONObject style = styleArray.optJSONObject(j, new JSONObject());
                if(style.isEmpty()){
                    cell.setCellStyle(defaultStyle);
                }else{
                    XSSFCellStyle cellStyle = createCellStyle(workbook,style);
                    cell.setCellStyle(cellStyle);
                }
                if(row_data.get(identifiers.optString(j, "")) instanceof String){
                    cell.setCellValue(row_data.optString(identifiers.optString(j, ""), ""));
                    cell.setCellType(org.apache.poi.ss.usermodel.CellType.STRING);
                }else if(row_data.get(identifiers.optString(j, "")) instanceof Integer){
                    cell.setCellValue(row_data.optInt(identifiers.optString(j, ""), 0));
                    cell.setCellType(org.apache.poi.ss.usermodel.CellType.NUMERIC);
                }else if(row_data.get(identifiers.optString(j, "")) instanceof Double){
                    cell.setCellValue(row_data.optDouble(identifiers.optString(j, ""), 0.0));
                    cell.setCellType(org.apache.poi.ss.usermodel.CellType.NUMERIC);
                }else if(row_data.get(identifiers.optString(j, "")) instanceof Boolean){
                    cell.setCellValue(row_data.optBoolean(identifiers.optString(j, ""), false));
                    cell.setCellType(org.apache.poi.ss.usermodel.CellType.BOOLEAN);
                }else{
                    cell.setCellValue(row_data.optString(identifiers.optString(j, ""), ""));
                    cell.setCellType(org.apache.poi.ss.usermodel.CellType.STRING);
                }
            }
        }

        for (int i = 0; i < identifiers.length(); i++) {
            sheet.autoSizeColumn(i, true);
        }
    }

    private XSSFCellStyle createCellStyle(XSSFWorkbook workbook, JSONObject styleJson) {

        XSSFColor background = getXSSFColor(styleJson.optString("background", "#ffffff"));
        XSSFColor foreground = getXSSFColor(styleJson.optString("foreground", "#333333"));
        boolean bold = styleJson.optBoolean("bold", false);
        boolean border = styleJson.optBoolean("border", true);
        String align = styleJson.optString("textAlign", "center");
        XSSFCellStyle style = workbook.createCellStyle();
        Font font = workbook.createFont();
        style.setAlignment( align.equals("center") ? HorizontalAlignment.CENTER : align.equals("left") ? HorizontalAlignment.LEFT : HorizontalAlignment.RIGHT);
        style.setWrapText(true);
        style.setVerticalAlignment(VerticalAlignment.CENTER);
        style.setFillPattern(FillPatternType.SOLID_FOREGROUND);

        if (border) {
            style.setBorderBottom(BorderStyle.THIN);
            style.setBorderTop(BorderStyle.THIN);
            style.setBorderLeft(BorderStyle.THIN);
            style.setBorderRight(BorderStyle.THIN);
            style.setBorderColor(BorderSide.BOTTOM, getXSSFColor("#333333"));
            style.setBorderColor(BorderSide.TOP, getXSSFColor("#333333"));
            style.setBorderColor(BorderSide.LEFT, getXSSFColor("#333333"));
            style.setBorderColor(BorderSide.RIGHT, getXSSFColor("#333333"));
        }else{
            style.setBorderBottom(BorderStyle.THIN);
            style.setBorderTop(BorderStyle.THIN);
            style.setBorderLeft(BorderStyle.THIN);
            style.setBorderRight(BorderStyle.THIN);
            style.setBorderColor(BorderSide.BOTTOM, getXSSFColor("#aaaaaa"));
            style.setBorderColor(BorderSide.TOP, getXSSFColor("#aaaaaa"));
            style.setBorderColor(BorderSide.LEFT, getXSSFColor("#aaaaaa"));
            style.setBorderColor(BorderSide.RIGHT, getXSSFColor("#aaaaaa"));
        }

        font.setBold(bold);
        style.setFont(font);

        if (background != null) {
            style.setFillForegroundColor(background);
        }

        if (foreground != null) {
            style.getFont().setColor(foreground);
        }

        return style;
    }

    public XSSFColor getXSSFColor(String rgb) {
        int red = Integer.parseInt(rgb.substring(1, 3), 16);
        int green = Integer.parseInt(rgb.substring(3, 5), 16);
        int blue = Integer.parseInt(rgb.substring(5, 7), 16);
        Color awtColor = new Color(red, green, blue);
        XSSFColor xssfColor = new XSSFColor();
        xssfColor
                .setRGB(new byte[] { (byte) awtColor.getRed(), (byte) awtColor.getGreen(), (byte) awtColor.getBlue() });
        return xssfColor;
    }

    public Resource convertByteToResource(byte[] data, String filename) {
        return new ByteArrayResource(data) {
            @Override
            public String getFilename() {
                return filename;
            }
        };
    }
}